      program texdot flood frequency
      implicit none
C-- COMPUTER PROGRAM FOR INTERACTIVE (SORT OF) ANALYSIS OF PEAK
C-- STREAMFLOW FREQUENCY IN TEXAS
C-- Distribution contains/needs the following files:
C--      txfreq.for : FORTRAN source code
C--      txfreq.exe : Executable
C--      cgrid.krg   : Gridded data set of the Climate Factors
C--      txfreq_info.txt : Text file containing more documentation
C--      rec664.txt  : Text file of the concurrent record matrix
C--      tx664.dat   : Text file of the core station data in log10
C--      error.fil   : Contains debugging diagnostics
C--   Upon successful execution, the program produces:
C--      ex_full.out : Full program output for example described in text.
C--      ex_sum.out  : Summary of program output for example in text.
C--      ex_sta.out  : Listing of stations selected and their respective 
C--                      data for example in text.

      integer MaxSites,MaxInd,MaxDatBase
      parameter (MaxSites = 50, MaxInd = 10, MaxDatBase = 700 )
C-- MaxSites = the maximum number of sites that can be selected
C-- MaxInd   = the maximum number of independent variables including
C--    the regression constant
C-- MaxDatBase = the maximum number of stations that can comprise
C--    the foundational data base.
      REAL Z, Sta, Uv, rhoc, lat, long   
      COMMON /ZZ/ Z(MaxDatBase,18),Sta(MaxSites),Uv(MaxInd),
     +rhoc(MaxSites,MaxSites),lat(MaxSites),long(MaxSites)
C
      REAL X, Xt, XtXinv, cov
      COMMON /XX1/ X(MaxSites,MaxInd), Xt(MaxInd,MaxSites),
     +XtXinv(MaxInd,MaxInd),cov(MaxSites,MaxSites)
C
      REAL Stano, Y, Wt, E, ET, C1
      COMMON /XY1/ Stano(MaxSites), Y(MaxSites,1),
     +Wt(MaxSites,MaxSites),E(MaxSites,1),ET(1,MaxSites),C1(1,1)
C
      CHARACTER*8 Xlabel(8), Ylabel(8), Pklabel(6)
      COMMON /YY1/ Xlabel, Ylabel, Pklabel
C
      REAL Bhat(MaxInd,1), Ak(6), ungaged_area, ungaged_slope,
     +Cf2, Cf25, Cf100
      COMMON /XX2/ Bhat, Ak, ungaged_area, ungaged_slope,
     +Cf2, Cf25, Cf100
C
      REAL Gamasq, reg_var, Atse, Atscov
      INTEGER Ne, Nsites
      COMMON /YY2/ Gamasq, reg_var, Ne, Nsites, Atse, Atscov
C
      CHARACTER*30 Siteid
      COMMON /YY3/ SITEID
      REAL alnd, alnm, alns, altd, altm, alts, da,dist,
     &     Similarity(MaxDatBase),distance(MaxDatBase),
     &     dr, ds, dh,
     &     ind_var_sd(1:8), sdbeta, reg_stdev,
     &     sum, Tbeta, tsave, Mcon(MaxDatBase,MaxDatBase),WLS,
     &     sd_dist,ave_dist
      real ungaged_lat,ungaged_long
C     real reg_skew,reg_kurtosis
      real alpha,theta,equiv_years,stdev_station
      real dd,alpha_area
      real ungaged_shape,ungaged_map,ungaged_P224
      INTEGER i,IndexArray(MaxDatBase),iout,ipk,iri,isave,istep,j
      INTEGER RecurIntArray(6), kpeak, total_stations
      integer similar_response,Ne_response
      character response*1,line*80,metric*1
      data RecurIntArray/ 2, 5, 10, 25, 50, 100 /
      external WLS
C-- Ak are used as the variance magnifiers of the magnitude and
C    frequency estimates of the station data.  These factors are 
C    the standard deviates for a normal distribution for exceedence
C    probabilities of .5, .2, .1, .02, and .01.  They are the K values
C    used to compute quantiles in X=M+KS, where M and S are the mean and
C    standard deviation. Assuming normality, the variance of X would be
C    (S**2/N)*(1+0.5*K**2). The K are appropriate for an LPIII if the
C    regional skew is assumed to be zero.  If K ne zero read the literature.
      data Ak/ 0.0, 0.84162, 1.28155, 1.75069, 2.05375, 2.32635 /

      data Xlabel/  'log(A)  ',
     &              'log(SL) ',
     &              'log(SH) ',
     &              'log(MAP)',
     &              'log(I24)',
     &              'C-fact2 ',
     &              'C-fact25',
     &              'C-fct100' /
      data Pklabel/ '   2-',
     &              '   5-',
     &              '  10-',
     &              '  25-',
     &              '  50-',
     &              ' 100-' /

      call STARTUP(Siteid,altd,altm,alts,alnd,alnm,alns,ungaged_area,
     &             ungaged_shape,ungaged_slope,ungaged_map,
     &             ungaged_P224,Nsites,alpha,theta,Cf2,Cf25,Cf100,
     &             similar_response,Ne_response,alpha_area,
     &             metric)

      call PRELIMINARIES(total_stations,Z,Mcon,ind_var_sd,
     *                   metric)

c  Compute distance between ungaged site and all gaging stations
c  based on area, precip intensity, and slope
      write(*,*) ' Calculating distances to all 664 stations'
      ungaged_lat = altd+(altm+(alts/60.))/60.
      ungaged_long = alnd+(alnm+(alns/60.))/60.
      sum = 0.0
      do 11 i = 1, total_stations
        call TASKER_DISTANCE(ungaged_lat,Z(i,2),
     &                       ungaged_long,Z(i,3),dist)
C-- Convert miles into kilometers
        if(metric.eq.'M') dist = dist*1.609344
        distance(i) = dist
        sum = sum + dist
 11   continue
      ave_dist = sum/total_stations
      do 12 i = 1,total_stations
         sd_dist = (distance(i)-ave_dist)**2+sd_dist
 12   continue
      sd_dist = sqrt( sd_dist / ( total_stations - 1 ) )
      do i = 1, total_stations
        da = (log10(ungaged_area)-Z(i,5))/ind_var_sd(1)
        if(ungaged_shape.gt.0.or.ungaged_slope.gt.0) then
          ds = (log10(ungaged_slope)-Z(i,8))/ind_var_sd(4)
          dh = (log10(ungaged_shape)-Z(i,9))/ind_var_sd(5)
        endif
        dd = distance(i)/sd_dist
        ipk = 10
C ** DISABLED FOR CLIMATE FACTOR REMOVAL
C ** The following two lines allow for different stations to be selected
C ** for different recurrence intervals--Eventually, we will use 
C ** the 25 climate factor.
c **        dr = (Cf25-Z(i,17))/ind_var_sd(7)
c **        dr = (Cf2-Z(i,16))/ind_var_sd(6)
c **        if ( ipk.GT.11 ) dr = (Cf25-Z(i,17))/ind_var_sd(7)
c **        if ( ipk.GT.13 ) dr = (Cf100-Z(i,18))/ind_var_sd(8)
         if(similar_response.eq.1) then
          if(Z(i,5).ge.(log10(ungaged_area)-alpha_area).and.
     &       Z(i,5).le.(log10(ungaged_area)+alpha_area)) then
                Similarity(i) = distance(i)
c               write(*,*) Similarity(i)
          else 
                Similarity(i) = 9999999
c              write(*,*) Similarity(i)
          endif
        elseif(similar_response.eq.2) then

C ** DISABLED FOR CLIMATE FACTOR REMOVAL
c **          Similarity(i) = SQRT(da**2+dr**2+ds**2+dh**2+dd**2)
           Similarity(i) = SQRT(da**2+ds**2+dh**2+dd**2)
        elseif(similar_response.eq.3) then
C ** DISABLED FOR CLIMATE FACTOR REMOVAL
c **          Similarity(i) = SQRT(da**2+dr**2)
           Similarity(i) = SQRT(da**2)
        elseif(similar_response.eq.4) then
          Similarity(i) = SQRT(da**2+dd**2)
        endif
      enddo
C-- Sort the Similarity(i) array and build an index array for
C    easy retrieval
      call INDEXX(total_stations,Similarity,IndexArray)
      write(*,*) ' BEGIN GLS REGRESSION'
      write(*,*) ' '
c BEGIN LOOP THROUGH ALL THE RECURRENCE INTERVALS
 400  CONTINUE
      DO 401 kpeak=1,6
        iri = RecurIntArray(kpeak)
        if ( iri.eq.  2 ) ipk = 10
        if ( iri.eq.  5 ) ipk = 11
        if ( iri.eq. 10 ) ipk = 12
        if ( iri.eq. 25 ) ipk = 13
        if ( iri.eq. 50 ) ipk = 14
        if ( iri.eq.100 ) ipk = 15

C PRELIMINARIES FOR REGRESSION
C Ne = number of independent variables to evaluated
C Uv(1:Ne) = array of the independent variables for the ungaged site
C Uv(1) is the regression constant
      if(Ne_response.eq.2) then

C ** REDUCED BY ONE FOR CLIMATE FACTOR REMOVAL
        Ne = 2
        Uv(1) = 1.0
        Uv(2) = log10(ungaged_area)
C ** DISABLED FOR CLIMATE FACTOR REMOVAL
c **       Uv(3) = Cf2
c **       if ( ipk.GT.11 ) Uv(3) = Cf25
c **       if ( ipk.GT.13 ) Uv(3) = Cf100
        Ylabel(1) = Xlabel(1)
C ** DISABLED FOR CLIMATE FACTOR REMOVAL
c **        Ylabel(2) = Xlabel(6)
c **        if ( ipk.GT.11 ) Ylabel(2) = Xlabel(7)
c **        if ( ipk.GT.13 ) Ylabel(2) = Xlabel(8) 
      else

C ** REDUCED BY ONE FOR CLIMATE FACTOR REMOVAL
        Ne = 4
        Uv(1) = 1.0
        Uv(2) = log10(ungaged_area)
        Uv(3) = log10(ungaged_slope)
        Uv(4) = log10(ungaged_shape)

C ** DISABLED FOR CLIMATE FACTOR REMOVAL
c **       Uv(5) = Cf2
c **       if ( ipk.GT.11 ) Uv(5) = Cf25
c **       if ( ipk.GT.13 ) Uv(5) = Cf100

        Ylabel(1) = Xlabel(1)
        Ylabel(2) = Xlabel(2)
        Ylabel(3) = Xlabel(3)
c        Ylabel(4) = Xlabel(6)
c        if ( ipk.GT.11 ) Ylabel(4) = Xlabel(7)
c        if ( ipk.GT.13 ) Ylabel(4) = Xlabel(8)   
      endif  

C Loop through the Nsites desired for the regression
      do 700 i = 1, Nsites
c Filling in the "1"'s for the regression constant X and Xt-transpose
        X(i,1) = 1.0
        Xt(1,i) = 1.0
c Filling in the rest of the independent variables
        X(i,2) = Z( IndexArray(i),5 )
        X(i,3) = Z( IndexArray(i),8 )
        X(i,4) = Z( IndexArray(i),9 )
        X(i,5) = Z( IndexArray(i),6 )
        X(i,6) = Z( IndexArray(i),7 )

C ** DISABLED FOR CLIMATE FACTOR REMOVAL
c **       X(i,7) = Z( IndexArray(i),16 )
c **       if ( ipk.GT.11 ) X(i,7) = Z( IndexArray(i),17 )
c **       if ( ipk.GT.13 ) X(i,7) = Z( IndexArray(i),18 )

c Filling in the Stano array with the character A8 station numbers
        Stano(i) = Z( IndexArray(i),1 )
c Filling in the dependent variable array
        Y(i,1) = Z( IndexArray(i),ipk )

C Building the rest of the Xt-transpose matrix
        do j = 2,Ne
          Xt(j,i) = X(i,j)
        enddo
C Loading the decimal latitude and longitude arrays
        lat(i) = Z( IndexArray(i),2 )
        long(i)= Z( IndexArray(i),3 )
 700  continue

C-- Build the cross-correlation matrix on the number of stations selected
C--  This subroutine uses the arrays lat and long as basis
      if(kpeak.eq.1) then
      call dist_correlation(Nsites,theta,alpha,lat,long,rhoc)
      endif
      reg_stdev    = 0.0 
C      reg_skew     = 0.0
C      reg_kurtosis = 0.0
C Compute the regional average moments,
      reg_stdev = 0.0 
      do i = 1, Nsites
        stdev_station = (Z(IndexArray(i),15)-Z(IndexArray(i),10))/Ak(6)
      reg_stdev = reg_stdev + stdev_station 

      enddo
      reg_stdev = reg_stdev / Nsites
      reg_var = reg_stdev**2
c      reg_skew = 0.0 / Nsites
c      reg_kurtosis = 0.0 / Nsites

C sta(i) = time sample error for site i
C Atse   = time sampling error (total at this point)
C Atscov = time sampling covariance (total at this point)
C equiv_years = the equivalent years weight: concurrent years divided by
C                  the product of the total years for each station
      Atse=0.0
      Atscov=0.0
      do 900 i = 1, Nsites
        do 901 j=1,i
          equiv_years=Mcon(IndexArray(i),IndexArray(j))
          equiv_years=equiv_years / Mcon(IndexArray(i),IndexArray(i))
          equiv_years=equiv_years / Mcon(IndexArray(j),IndexArray(j))

C-- Covariance model, if i=j then the regional variance is plugged in
C--  In the original released source reg_var was squared, but this is
C--  already a variance so the square is not needed.  This was fixed
C--  on May 23, 2001 by WHA after Robert Dusenbury <robd@aquaterra.com>
C--  reported problem to Gary Tasker.  This correction appears consistent
C--  with a March 11, 2003 check on the Arkansas code (p4-6) by WHA.
          Cov(i,j)=rhoc(i,j)*reg_var
          Cov(i,j)=Cov(i,j)*(1.+rhoc(i,j)*.5*Ak(ipk-9)**2)*equiv_years
          if(Cov(i,j).lt.0.0) then
           write(*,*) 'Cov(i,j) = ',i,j,Cov(i,j)
           write(*,*) 'rhoc(i,j) = ',i,j,rhoc(i,j)
           write(*,*) 'equiv_years = ',equiv_years
           write(*,*) 'ERROR--Cholesky Decomp requires >= 0'
           stop
         endif
C-- Making the variance-covariance matrix symmetric
           Cov(j,i)=Cov(i,j)

C-- Set the time sampling error of station i to the variance on the diagonal
C--   and compute the total time sampling error
         if(i.eq.j) then
            Sta(i) = Cov(i,i)
            Atse   = Atse + sta(i)
         else
            Atscov = Atscov + Cov(i,j)
         endif
 901    continue
 900  continue
c END REGRESSION PRELIMINARIES


C STEP-BACKWARD GENERALIZED LEAST SQUARES REGRESSION
C istep = step of the regression
      istep = 0
 1000 CONTINUE
      CALL SECANT(WLS)
      istep = istep + 1
    
      CALL OUTPUT(istep,ipk-9,X,Xt,XtXinv,Wt,Bhat,Uv,Y,Sta,
     &  Stano,Xlabel,Ylabel,Pklabel,ungaged_area,ungaged_slope,
     &  ungaged_shape,Cf2,Cf25,Cf100,GamaSq,Atse,Atscov,Ne,Nsites,
     &  Siteid,metric,similar_response)

C-- Compute T for each Beta in model
C--    do step-backward by dropping all variables with T < 2,
C--    which is a p-value of about 0.30 or smaller depending upon
C--    degrees of freedom on the T-distribution.  Tsave is used to insure
C--    that in the case where two or more variables have T < 2, then the
C--    variable having the smallest T is preferentially dropped first.
      Tsave = 2.0
      isave = 100
C-- Loop through all the variables except the regression constant (i.eq.1)
C--   Except for the Texas implementation here, we want drainage area to 
C--   always be present in the regression equation for reasons described on
C--   page 5, second column, first complete paragraph.  Hence, we start
C--   the iteration at 3 instead of 2.  This explanation for the starting
C--   at 3 was not added until October 15, 2001 by WHA after Robert Dusenbury
C--   of Aquaterra <robd@aquaterra.com> reported differences between the
C--   Texas and the North Carolina code via email.
      do 1100 i = 3, Ne
        sdbeta = SQRT(XtXinv(i,i))
        Tbeta = ABS(Bhat(i,1)/sdbeta)
C-- Besides looking to drop Tbetas less than 2, we also want the smallest
c    Tbeta less than 2 for each step in the step-wise regression.
        if ( Tbeta .lt. 2 .and. Tbeta .lt. Tsave ) then
          Tsave = Tbeta
          isave = i
        endif
 1100 continue

C-- When isave equals Ne, then we want to drop the last independent
C--  variable, therefore, all we do is ignore the last column of X
      if ( isave.EQ.Ne ) then
        Ne = Ne - 1
        GOTO 1000
      endif

C-- Perform preliminaries before recalling the regression algorithms
      if ( isave.LT.Ne ) then
C-- The count of the number of independent variables decrements by one
        Ne = Ne - 1
C-- The labels on the Y need shifting
        do 1150 j = isave, Ne
          Ylabel(j-1) = Ylabel(j)
 1150   continue

C-- Redefining (removing dropped variable) the X matrix and its transpose
        do 1200 i = 1, Nsites
          do 1160 j = isave, Ne
            X(i,j) = X(i,j+1)
            Xt(j,i) = X(i,j)
 1160     continue
 1200   continue

C-- Shift ungaged site parameters because one parameter was just dropped
        do 1250 j = isave, Ne
          Uv(j) = Uv(j+1)
 1250   continue
        goto 1000
      endif

C OUTPUT THE FINAL REGRESSION SUMMARY, isave remains large (isave = 100)
      iout = 99
      CALL OUTPUT(iout,ipk-9,X,Xt,XtXinv,Wt,Bhat,Uv,Y,Sta,
     &  Stano,Xlabel,Ylabel,Pklabel,ungaged_area,ungaged_slope,
     &  ungaged_shape,Cf2,Cf25,Cf100,GamaSq,Atse,Atscov,Ne,Nsites,
     &  Siteid,metric,similar_response)

C FINISHED THE RECURRENCE INTERVAL LOOP
  401 continue 

C-- Placing the regression statistics documentation into output file
        write(*,*) ' '
        write(*,*) ' END OF ANALYSIS'

          write(17,'(1x,70a)') ('-',i=1,70)         
          write(17,*) 'REGRESSION DOCUMENTATION (as follows)'
          open(12,file='txfreq_info.txt',status='OLD',err=995)
 996      read(12,'(a80)',end=997) line
          write(17,'(1x,a80)') line
          goto 996
 995      write(*,*) "ERROR--Could not find 'txfreq_info.txt' to load"
C-- The txfreq_info.txt file contains regression documentation, this file
C-- needs to be in the same directory as the executable
          goto 998
 997    close(12)
        close(17)
        close(14)

C-- Generating an ASCII file of the selected stations.
      write(*,*) ' '
 998  write(*,'(a,i2)') '   Writing the data for the ',Nsites
        write(11,*) 'DATA FOR STATIONS SELECTED FOR GLS REGRESSION'
        write(11,*) 'SITE ID = ',SITEID
        write(11,*) 'Number of sites selected = ',Nsites
9020   format(' Similarity',2x,'Distance',2x,'Station',3x,'Latitude',2x,
     &        'Longitude',1x,'EqYears',3x,'CDA',9x,'MAP',8x,'P224',6x,
     &        'Slope',6x,'Shape',6x,'Q2',9x,'Q5',8x,'Q10',8x,'Q25',8x,
     &        'Q50',7x,'Q100',10x,'C2',8x,'C25',8x,'C100')
9021   format('   (--)    ',2x,' (mile) ',2x,'  (--) ',3x,'(DecDeg)',2x,
     &        ' (DecDeg)',1x,' (yrs) ',3x,'(sqmile)',4x,'(in)',7x,
     &        '(in)',6x,'(ft/mi)',4x,'(--)',7x,'(cfs)',6x,'(cfs)',5x,
     &        '(cfs)',6x,'(cfs)',6x,
     &        '(cfs)',5x,'(cfs)',9x,'(--)',7x,'(--)',7x,'(--)')
9022   format('   (--)    ',2x,' (mile) ',2x,'  (--) ',3x,'(DecDeg)',2x,
     &        ' (DecDeg)',1x,' (yrs) ',2x,'(logsqmile)',2x,'(login)',4x,
     &        '(login)',3x,'(logft/mi)',1x,'(log--)',4x,'(logcfs)',3x,
     &        '(logcfs)',2x,'(logcfs)',3x,'(logcfs)',3x,'(logcfs)',2x,
     &        '(logcfs)',5x,'(--)',7x,'(--)',7x,'(--)')
        write(11,9020)
        write(11,9021)

        do i = 1, Nsites
          write (11,9010) Similarity(IndexArray(i)),
     &                    Distance(IndexArray(i)),
     &                    (Z(IndexArray(i),j),j=1,4),
     &                    (10**Z(IndexArray(i),j),j=5,15),
     &                    (Z(IndexArray(i),j),j=16,18)
        enddo 
		  
        write(11,*) '   '
        write(11,*) '   '
        write(11,*) 'LOG-DATA FOR STATIONS SELECTED FOR GLS REGRESSION'
        write(11,*) 'SITE ID = ',SITEID
        write(11,*) 'Number of sites selected = ',Nsites
        write(11,9020)
        write(11,9022)
        do i = 1, Nsites
        write (11,9015) Similarity(IndexArray(i)),
     &                    Distance(IndexArray(i)),
     &                    (Z(IndexArray(i),j),j=1,18)
        enddo 		  
		  
        write(11,*) ' '
        write(11,*) ' '
        write(11,*) 'EXPLANATIONS'
        write(11,*) 'Similarity -- see discussion in body of report'
        write(11,*) 'Distance -- distance between site and station'
        write(11,*) 'Latitude -- Decimal degrees and not DDMMSS'
        write(11,*) 'Longitude -- Decimal degrees and not DDMMSS'
        write(11,*) 'EqYears -- Equivalent years of record for station'
        write(11,*) 'CDA -- Contributing drainage area'
        write(11,*) 'MAP -- Mean Annual Precip. (1951-80) for basin'
        write(11,*) 'P224 -- 2-year 24-hour Precip. (P-40) for basin'
        write(11,*) 'Slope -- Stream slope @ 1/100k scale (see report)'
        write(11,*) 'Shape -- Basin Shape Factor (length**2 / CDA)'
        write(11,*) 'QT -- T-year Peak Discharge'
        write(11,*) 'C2 -- 2-year Climate Factor (see report)'
        write(11,*) 'C25 -- 25-year Climate Factor (see report)'
        write(11,*) 'C100 -- 100-year Climate Factor (see report)'
        close(11)

c      write(*,*) '  Successful Exit '
c      write(*,*) 'Hit any key <enter> to continue'
c      read(*,*) response
      stop
 9005 format(3F10.4,f5.0,14F10.4)
 9010 format(1x,F7.4,1x,f10.2,1x,f10.0,1x,2(F9.3,1x),1x,f5.0,1x,
     &       14(f10.2,1x))
 9015 format(1x,F7.4,1x,f10.2,1x,f10.0,1x,2(F9.3,1x),1x,f5.0,1x,
     &       14(F10.4,1x))
      end



      SUBROUTINE INDEXX(N,ARRIN,INDEXARRAY)
c Subroutine INDEXX produces an index array from array ARRIN(1:N)
c  such that ARRIN(INDX(J)) is in ascending order for
c  J=1,2,..,N. The input quantities ARRIN and N are not changed
c      (ref. Numerical Recipes, p. 233)
      implicit none
      integer MaxDatBase
      parameter (MaxDatBase = 700)
      integer INDEXARRAY(1:MaxDatBase)
      real ARRIN(1:MaxDatBase)
      real q
      integer i,indxt,ir,j,l,N
C
      do j = 1, N
        INDEXARRAY(j) = j
      enddo
      l = N/2 + 1
      ir = N
 200  continue
      if ( l.GT.1 ) then
        l = l - 1
        indxt = INDEXARRAY(l)
        q = ARRIN(indxt)
      else
        indxt = INDEXARRAY(ir)
        q = ARRIN(indxt)
        INDEXARRAY(ir) = INDEXARRAY(1)
        ir = ir - 1
        if ( ir.EQ.1 ) then
          INDEXARRAY(1) = indxt
          return
        endif
      endif
      i = l
      j = l + l
 300  continue
      if ( j.LE.ir ) then
        if ( j.LT.ir ) then
          if ( ARRIN(INDEXARRAY(j)).LT.ARRIN(INDEXARRAY(j+1)) ) THEN
            j = j + 1
          endif
        endif
        if ( q.LT.ARRIN(INDEXARRAY(j)) ) then
          INDEXARRAY(i) = INDEXARRAY(j)
          i = j
          j = j + j
        else
          j = ir + 1
        endif
        GOTO 300
      endif
      INDEXARRAY(i) = indxt
      goto 200
      end


C-- FUNCTION COMPUTES THE REGRESSION COEFFICIENTS AND
C-- THE WEIGHTED SSE (SUM OF SQUARE ERROR)
      REAL FUNCTION WLS(GAMA2)
      implicit none
      integer MaxSites, MaxInd, MaxDatBase
      parameter (MaxSites = 50, MaxInd = 10, MaxDatBase = 700)
      REAL GAMA2, det, XtX(MaxInd,MaxInd), work(MaxInd,MaxSites),
     +work2(MaxInd,MaxSites),work1(1,MaxSites)
      INTEGER k
      REAL Z, Sta, Uv, rhoc, lat, long   
      COMMON /ZZ/ Z(MaxDatBase,18),Sta(MaxSites),Uv(MaxInd),
     +rhoc(MaxSites,MaxSites),lat(MaxSites),long(MaxSites)
      REAL X, Xt, Xtxinv, cov
      COMMON /XX1/ X(MaxSites,MaxInd), Xt(MaxInd,MaxSites),
     +Xtxinv(MaxInd,MaxInd),
     +cov(MaxSites,MaxSites)
      REAL Stano, Y, Wt, E, ET, C1
      COMMON /XY1/ Stano(MaxSites), Y(MaxSites,1),
     +Wt(MaxSites,MaxSites),E(MaxSites,1),ET(1,MaxSites),C1(1,1)
      CHARACTER*8 Xlabel, Ylabel, Pklabel
      COMMON /YY1/ Xlabel(8), Ylabel(8), Pklabel(6)
      REAL Bhat(MaxInd,1), Ak(6), ungaged_area, ungaged_slope, Cf2,
     +Cf25, Cf100
      COMMON /XX2/ Bhat, Ak, ungaged_area, ungaged_slope,
     +Cf2, Cf25, Cf100
      REAL Gamasq, reg_var, Atse, Atscov
      INTEGER Ne, Nsites
      COMMON /YY2/ Gamasq, reg_var, Ne, Nsites, Atse, Atscov
C      CHARACTER*30 Siteid
C      COMMON /YY3/ SITEID

C-- Bhat = {Xt*Cov'*X}*Xt*Cov'*Y, where ' denotes the inverse
C-- Compute the weighting matrix Wt by first generating the estimated 
C--   GLS covariance matrix, Cov(k,k) by updating the diagonal with the
C--   sum of the model error gama2 estimated from first call of SECANT and
C--   the station time sampling error.
      do k = 1, Nsites
        Cov(k,k) = GAMA2+Sta(k)
        if(Cov(k,k).le.0.0) then
          write(*,*) 'Diagonal of variance-covariance matrix is <= 0'
          write(*,*) 'Location (k,k) of ',k,',',k
          stop
        endif
      enddo
c      write(*,*) 'STATUS--Inverting the Variance-Covariance Matrix'
      call INVERT(Nsites,MaxSites,det,Wt,Cov)
C-- Multiply Xt-transpose by the inverted covariance array (Wt)
C--   and produce work
      CALL MLTPLY(work,Xt,Wt,Ne,Nsites,Nsites,MaxInd,MaxInd,MaxSites)
C-- Multiply Xt*Wt (work) by X to produce XtX
      CALL MLTPLY(XtX,work,X,Ne,Nsites,Ne,MaxInd,MaxInd,MaxSites)

C-- COMPUTE THE REGRESSION COEFFICIENTS
C-- Invert the Xt*Wt-1*X matrix
c      write(*,*) 'STATUS--Inverting the Ind. Variable Vector'
      CALL INVERT(Ne,MaxInd,det,XtXinv,XtX)
C-- Multiply the inverted variance-covariance matrix by Xt
      CALL MLTPLY(work,XtXinv,Xt,Ne,Ne,Nsites,MaxInd,MaxInd,MaxInd)
C-- Multiply the results of above the Wt matrix
      CALL MLTPLY(work2,work,Wt,Ne,Nsites,Nsites,MaxInd,MaxInd,
     &                                                   MaxSites)
c      write(*,*) 'STATUS--Estimating the Coefficients'
C-- Estimate the coefficients Bhat
      CALL MLTPLY(Bhat,work2,Y,Ne,Nsites,1,MaxInd,MaxInd,MaxSites)

C-- ERROR ESTIMATION
C-- Determine the Error matrix E from by first estimating E as
C--   product of the independent variables and the coefficients
      CALL MLTPLY(E,X,Bhat,Nsites,Ne,1,MaxSites,MaxSites,MaxInd)
C-- The fill the E matrix with the residuals and make the transpose
      do k = 1, Nsites
        E(k,1)=Y(k,1)-E(k,1)
        Et(1,k)= E(k,1)                            
      enddo
C-- Multiply Et by Wt to produce a vector of error weights
      CALL MLTPLY(work1,Et,Wt,1,Nsites,Nsites,1,1,MaxSites)
C-- Multiply work1 by the Errors to produce a single value of error
      CALL MLTPLY(C1,work1,E,1,Nsites,1,1,1,MaxSites)
C-- WLS is the weighted SSE sum of square error
      WLS = (Nsites-Ne)/C1(1,1) - 1.
c      write(*,*) 'STATUS--Exiting WLS'
      return
      end


      SUBROUTINE OUTPUT(IOUT,IPK,X,Xt,XtXinv,Wt,Bhat,Uv,Y,Sta,
     &  Stano,Xlabel,Ylabel,Pklabel,ungaged_area,ungaged_slope,
     &  ungaged_shape,Cf2,Cf25,Cf100,GamaSq,Atse,Atscov,Ne,Nsites,
     &  Siteid,metric,similar_response)
      implicit none
      integer MaxSites, MaxInd, MaxDatBase
      parameter (MaxSites = 50, MaxInd = 10, MaxDatBase = 700)
      REAL cookd, delres, hatdig, hmax, press, log_pred,
     &     ungaged_pred, pv4, resid, samerr, sdbeta,  sepc, exp1,
     &     cl50,cu50,cl67,cu67,cl90,cu90, sepu
      REAL hat(MaxSites,MaxSites), hats(MaxSites,MaxSites)
      REAL ssam, stdres, STUTP, sum, tbeta, hatdigtest, cooktest,
     &     tstat, tv4, varres, vpu, work1(MaxSites,1),
     &     work3(MaxInd,MaxSites), work2(MaxInd,1), STUTX
      INTEGER i, IOUT, IPK, j, ndf
      REAL Sta(MaxSites),Uv(MaxInd), maxlev
      REAL X(MaxSites,MaxInd), Xt(MaxInd,MaxSites),
     +     XtXinv(MaxInd,MaxInd)
      REAL Stano(MaxSites), Y(MaxSites,1), Wt(MaxSites,MaxSites)
      CHARACTER*8 Xlabel(8), Ylabel(8), Pklabel(6)
      REAL Bhat(MaxInd,1), ungaged_area, ungaged_slope,
     & ungaged_shape, Cf2, Cf25, Cf100
      REAL Gamasq, Atse, Atscov, MEV, MEVpc
      INTEGER Ne, Nsites,similar_response
      CHARACTER*30 Siteid,metric*1
      real Arhoc

      if(IOUT.LE.0) then
        write(*,*) 'ERROR--Improper call of OUTPUT subroutine. IOUT<=0'
        stop
      endif
 8899 format(a,a,i1)
 9005 FORMAT (a,a5,a,i2) 
 9010 FORMAT (a,a5,a)
      if ( IOUT.LT.99 ) then
        if(IOUT.eq.1) then
C          write(*,*) ' '
          write(17,*) ' '
C          write(*,8899)  '   Number of stations in ',
C     &                          'regression = ',Nsites
          write(17,8899)  '   Number of stations in ',
     &                          'regression = ',Nsites
          if(similar_response.eq.1) then
 9002     format('   Stations chosen by ',a,/,20x, 
     &'(see text for details)',/)
C             write(*,9002) 'PROXIMITY (1) alpha = 1.5 log10'
            write(17,9002) 'PROXIMITY (1) alpha = 1.5 log10'
          elseif(similar_response.eq.2) then
C             write(*,9002) 'SIMILARITY (2) CDA, Cf25, SL, SH, ', 
C     &                     'Distance'
            write(17,9002) 'SIMILARITY (2) CDA, Cf25, SL, SH, ',
     &                     'Distance'
          elseif(similar_response.eq.3) then
C             write(*,9002) 'SIMILARITY (3) CDA, Cf25'
            write(17,9002) 'SIMILARITY (3) CDA, Cf25'
          elseif(similar_response.eq.4) then
C             write(*,9002) 'SIMILARITY (4) CDA, Distance'
            write(17,9002) 'SIMILARITY (4) CDA, Distance'
          endif
        endif
C-- This displays the regression step header
C        write(*,9005) '  STEPWISE GLS REGRESSION FOR ',
C     &                     Pklabel(IPK),'YEAR PEAK === STEP ',
C     &                     IOUT
        write(17,9005)  '  STEPWISE GLS REGRESSION FOR ',
     &                     Pklabel(IPK), 'YEAR PEAK === STEP ',
     &                     IOUT  
      else
C-- This displays the final equation header
        write(*,'(1x,70a)') ('-',i=1,70)
        write(14,'(1x,70a)') ('-',i=1,70)
        write(17,'(1x,70a)') ('-',i=1,70)
C        write(*,9010)  '  FINAL GLS REGRESSION FOR ',Pklabel(IPK),
C     &            'YEAR PEAK === FINAL EQUATION'
        write(17,9010) '  FINAL GLS REGRESSION FOR ',PKlabel(IPK),
     &            'YEAR PEAK === FINAL EQUATION'
      endif

 9015 FORMAT ('  Regression Coefficients (Betas)',/,
     & '  Variable      Beta       Std Error   T for Ho:Beta=0'
     & ,'  P-Value > |T|')
C      write(*,9015)
      write(17,9015)
      
 9020 FORMAT ('  Constant',3x,f10.5,3x,f7.4,9x,f8.4,9x,f6.4)
C-- REGRESSION CONSTANT
C-- The variance of the Beta's sits along the diagonal of XtXinv
      sdbeta = SQRT(XtXinv(1,1))
C-- Divide Beta by standard deviation to "Studentize" or standardize
      Tbeta = Bhat(1,1)/sdbeta
C      write(*,9020)    Bhat(1,1),sdbeta,Tbeta
      write(17,9020) Bhat(1,1),sdbeta,Tbeta    

C-- LOOP THROUGH THE REMAINING BETAS
      do 50 i = 2, Ne
C-- Again variance of the Beta's sits along the diagonal of XtXinv
        sdbeta = SQRT(XtXinv(i,i))
        Tbeta = Bhat(i,1)/sdbeta
        ndf = Nsites - Ne
        tv4 = ABS(Tbeta)
C-- Calculate the p-value of the magnitude of the Tbeta
        pv4 = 2.0*STUTP(-tv4,ndf)
        if ( pv4.LT.0.0001 ) pv4 = .0001
 9025 FORMAT (2X,a8,3x,f10.5,3x,f7.4,9x,f8.4,12x,f6.4)
C        write(*,9025) Ylabel(i-1), Bhat(i,1),sdbeta,Tbeta,pv4
        write(17,9025) Ylabel(i-1),Bhat(i,1),sdbeta,Tbeta,pv4
        
 50   continue
C        write(*,*)  ' '
        write(17,*) ' '
C-- If not on the final equation return to main program
      if(IOUT.LT.99) return
 
 9030 FORMAT (/,'  High Leverage or Highly Influencial Observations',
     &        ' (log10 if applicable)',
     &        /,'     Station',t15,'   Obser',t27,'  Pred',t39,
     &        ' Std Resid',t51,' Leverage',t62,' Cook-D')
C      write(*,9030)
      write(17,9030)

C WRITE PREDICTED VALUES ETC.
      CALL MLTPLY(work1,X,Bhat,Nsites,Ne,1,MaxSites,
     &                                              MaxSites,MaxInd)
      ssam   = 0.0
      press  = 0.0
      hmax   = 0.0
      maxlev = 0.0
      CALL MLTPLY(work3,XtXinv,Xt,Ne,Ne,Nsites,MaxInd,MaxInd,MaxInd)
      CALL MLTPLY(hats,X,work3,Nsites,Ne,Nsites,MaxSites,MaxSites,
     &                                                       MaxInd)
      CALL MLTPLY(hat,hats,Wt,Nsites,Nsites,Nsites,MaxSites,
     &                                            MaxSites,MaxSites)
      do 60 i = 1, Nsites
C-- Estimation of Y for station i
        log_pred = work1(i,1)
C-- Calculate residual
        resid = Y(i,1) - log_pred
C-- The sampling error variance is along diagonal
        samerr = hats(i,i)
C-- Sum up the station sampling errors
        ssam = ssam + samerr
C-- Searching for the maximum sampling error variance
        if ( samerr.GT.hmax ) hmax = samerr
C-- hatdig is the leverage of site i, which sits along diagonal
        hatdig = hat(i,i)
        if(hatdig.ge.maxlev) maxlev = hatdig
        delres = resid/(1.0-hatdig)
        press = press + delres**2

C-- Residual variance = (model error variance) +
C--                     (time-sampling error variance for station i) -
C--                     (sampling error variance of the estimation)
        varres = GamaSq + Sta(i) - samerr
        stdres = resid/SQRT(varres)
        cookd = stdres**2*samerr/(Ne*(varres))

C-- Tasker's rules-of-thumb on flagging potential 'Outliers'
        hatdigtest = 2.0 * Ne / Nsites
        cooktest = 4.0 / Nsites
        if ( hatdig.GT.hatdigtest .OR. cookd.GT.cooktest ) then
 9035 FORMAT (1X,F10.0,4F12.5,f10.5)
C          write(*,9035)  Stano(i),Y(i,1),log_pred,stdres,hatdig,cookd
          write(17,9035) Stano(i),Y(i,1),log_pred,stdres,hatdig,cookd
        endif
 60     continue
C        write(*,*) ' ' 
        write(17,*) ' '

C-- TOTAL REGRESSION DIAGNOSTICS
C-- Compute the 'total' regression errors etc . . .
C-- ssam   = Average Sampling Error Variance
C-- GamaSq = Model Error Variance
C-- Atse   = Average Time-Sampling Error Variance
C-- Atscov = Average Cross-Covariance
C-- Arhoc  = Average Cross-Correlation Coefficient
C-- MEV    = Mean error variance of total regression
      ssam   = ssam   / Nsites
      press  = press  / Nsites
      Atse   = Atse   / Nsites
      Atscov = Atscov / ( Nsites / 2.0 * (Nsites - 1 ) )
      Arhoc  = Atscov / Atse
      MEV = ssam + GamaSq
 
C      write(*,*)   ' TOTAL REGRESSION ERRORS AND DIAGNOSTICS',
C     &'              SqRoot (log10)'
      write(17,*)  ' TOTAL REGRESSION ERRORS AND DIAGNOSTICS',
     &'              SqRoot (log10)'
 9040 FORMAT ('      Average Sampling Error Variance: ',F10.4,
     &        ' --->',f8.4,/,
     &        '      Maximum Sampling Error Variance: ',f10.4,
     &        ' --->',f8.4,/,
     &        '      Model Error Variance (gamma**2): ',F10.4,
     &        ' --->',f8.4,/,
     &        '      Std. Error Variance of Equation: ',F10.4,
     &        ' --->',f8.4,/,
     &        '      Std. Error Variance of Equation: ',F7.2,
     &        ' percent',/,
     &        '    Ave. Time-Sampling Error Variance: ',F10.4,
     &        ' --->',f8.4,/,
     &        '                              PRESS/N: ',f10.4,/,
     &        '            Average Cross Correlation: ',f10.4,/,
     &        '                     Maximum Leverage: ',f10.4)
      MEVpc = 100.*SQRT(EXP(MEV*5.302)-1.)
C      write(*,9040)  ssam, sqrt(ssam), hmax, sqrt(hmax),
C     &               GamaSq, sqrt(GamaSq), MEV, sqrt(MEV),
C     &               MEVpc,
C     &               Atse, sqrt(Atse), press, Arhoc, maxlev
      write(17,9040) ssam, sqrt(ssam), hmax, sqrt(hmax),
     &               GamaSq, sqrt(GamaSq), MEV, sqrt(MEV),
     &               MEVpc,
     &               Atse, sqrt(Atse), press, Arhoc, maxlev

C-- UNGAGED SITE ERRORS
      log_pred = 0.0
C-- Calculate the ungaged site estimation, log10 then real-space
      do i = 1, Ne
        log_pred = log_pred + Bhat(i,1)*Uv(i)
      enddo
      ungaged_pred = 10**log_pred

C-- Calculate the error variance specific to the ungaged site
C--  in relation to the other sites in the regression model
      do 100 i = 1, Ne
        sum = 0.0
        do 90 j = 1, Ne
          sum = sum + XtXinv(i,j)*Uv(j)
 90     continue
        work2(i,1) = sum
 100  continue
      sum = 0.0
      do 120 j = 1, Ne
        sum = sum + Uv(j)*work2(j,1)
 120  continue
      sepu = sum
      vpu = GamaSq + sepu
C-- Error to percent error, Tasker's neat transformation
      sepc = 100.*SQRT(EXP(vpu*5.302)-1.)
C-- For 90 percent limits, tstat = SQRT(vpu)*1.65 for large df
      exp1  = STUTX(1-(0.1/2),ndf)         
      tstat = SQRT(vpu)*exp1
      cu90  = 10**(tstat + log_pred)
      cl90  = 10**(log_pred - tstat)
C-- For 67 percent limits, tstat = SQRT(vpu)*0.974 for large df
      exp1  = STUTX(1-(0.33/2),ndf)
      tstat = SQRT(vpu)*exp1
      cu67  = 10**(tstat + log_pred)
      cl67  = 10**(log_pred - tstat)
C-- For 50 percent limits, tstat = SQRT(vpu)*0.675 for large df
      exp1  = STUTX(1-(0.5/2),ndf)
      tstat = SQRT(vpu)*exp1
      cu50  = 10**(tstat + log_pred)
      cl50  = 10**(log_pred - tstat)
 
 9022 format('  UNGAGED SITE ESTIMATION ',a)
      if(metric.eq.'M') then     
         write(*,9022)  '(cubic meters per second)'
         write(14,9022)  '(cubic meters per second)'
         write(17,9022) '(cubic meters per second)'
      else
         write(*,9022)  '(cubic feet per second)'
         write(14,9022)  '(cubic feet per second)'
         write(17,9022) '(cubic feet per second)'
      endif

      write(*,'(3x,a)')  '   T-year    Estimation'  
      write(17,'(3x,a)') '   T-year    Estimation'
 9045 FORMAT (2x,a5,'year',f12.2,/)
      write(*,9045) Pklabel(IPK),ungaged_pred
      write(14,9045) Pklabel(IPK),ungaged_pred
      write(17,9045) Pklabel(IPK),ungaged_pred
      write(*,*)    ' ESTIMATION ERRORS AND LIMITS           ',
     &'              Sqroot (log10)'
      write(14,*)    ' ESTIMATION ERRORS AND LIMITS           ',
     &'              Sqroot (log10)'
      write(17,*)   ' ESTIMATION ERRORS AND LIMITS           ',
     &'              Sqroot (log10)'     
 9046 FORMAT ('          Standard Error of Estimation: ',F7.2,
     &        ' percent',/,
     &        '      Std Error Variance of Estimation: ',f9.4,
     &        ' --->',f8.4,/,
     &        ' Sampling Error Variance of Estimation: ',f9.4,
     &        ' --->',f8.4)   
      write(*,9046)  sepc, vpu, sqrt(vpu), sepu, sqrt(sepu)
      write(14,9046)  sepc, vpu, sqrt(vpu), sepu, sqrt(sepu)
      write(17,9046) sepc, vpu, sqrt(vpu), sepu, sqrt(sepu)

C--Test and flag for large standard error of estimation percentages
      if(sepc.ge.55) then
C        write(*,9077) Pklabel(IPK)
        write(17,9077) Pklabel(IPK)
      endif
 9077 format(/,'   HELPFUL HINT--',/,
     &       '   Testing during development of program indicated that',
     &       ' percent errors',/,
     &       '   in excess of about 55% might be higher than ',
     &       'necessary.  A NEW',/,
     &       '   regression equation generated with different options ',
     &       'often resulted',/,
     &       '   in a smaller precent error.  An inference could be ',
     &       'made that the',/,
     &       '   smaller percent error regression produces a more ',
     &       'accurate',a5,'year',/,
     &       '   peak streamflow estimate than the regression shown ',
     &       'here.')
      write(*,*)  ' '
      write(*,*)  ' Estimation Limits     Upper         Lower' 
      write(*,*)  '     90-percent   ',cu90,cl90,'   Limits are two-'
      write(*,*)  '     67-percent   ',cu67,cl67,'   tailed, or ___%' 
      write(*,*)  '     50-percent   ',cu50,cl50,'   bounding limits.'
      write(14,*)  ' '
      write(14,*)  ' Estimation Limits     Upper         Lower' 
      write(14,*)  '     90-percent   ',cu90,cl90,'   Limits are two-'
      write(14,*)  '     67-percent   ',cu67,cl67,'   tailed, or ___%' 
      write(14,*)  '     50-percent   ',cu50,cl50,'   bounding limits.'
      write(17,*) ' '
      write(17,*) ' Estimation Limits     Upper         Lower' 
      write(17,*) '     90-percent   ',cu90,cl90
      write(17,*) '     67-percent   ',cu67,cl67
      write(17,*) '     50-percent   ',cu50,cl50
C      write(*,'(/,a)')   '  UNGAGED SITE SUMMARY'
      write(17,'(/,a)')  '  UNGAGED SITE SUMMARY'
C ** DISABLED FOR CLIMATE FACTOR REMOVAL
c ** 9047 FORMAT ('        Ungaged Site Name: ',a30,/,
c **     &        '             Drainage Area (mi^2): ',F8.2,/,
c **     &        '             Stream Slope (ft/mi): ',F8.2,/,
c **     &        '                 Basin Shape (--): ',F8.2,/,
c **     &        '       2-year Climate Factor (--): ',f10.4,/,
c **     &        '      25-year Climate Factor (--): ',f10.4,/,
c **     &        '     100-year Climate Factor (--): ',f10.4)
c ** 9048 FORMAT ('        Ungaged Site Name: ',a30,/,
c **     &        '             Drainage Area (km^2): ',F8.2,/,
c **     &        '              Stream Slope (m/km): ',F8.2,/,
c **     &        '                 Basin Shape (--): ',F8.2,/,
c **     &        '       2-year Climate Factor (--): ',f10.4,/,
c **     &        '      25-year Climate Factor (--): ',f10.4,/,
c **     &        '     100-year Climate Factor (--): ',f10.4)    
 9047 FORMAT ('        Ungaged Site Name: ',a30,/,
     &        '             Drainage Area (mi^2): ',F8.2,/,
     &        '             Stream Slope (ft/mi): ',F8.2,/,
     &        '                 Basin Shape (--): ',F8.2)
 9048 FORMAT ('        Ungaged Site Name: ',a30,/,
     &        '             Drainage Area (km^2): ',F8.2,/,
     &        '              Stream Slope (m/km): ',F8.2,/,
     &        '                 Basin Shape (--): ',F8.2)     
       if(metric.eq.'M') then
C ** DISABLED FOR CLIMATE FACTOR REMOVAL
C **         write(17,9048) Siteid, ungaged_area, ungaged_slope,
C **     &                  ungaged_shape, Cf2, Cf25, Cf100
         write(17,9048) Siteid, ungaged_area, ungaged_slope,
     &                  ungaged_shape
       else
C ** DISABLED FOR CLIMATE FACTOR REMOVAL
C **        write(17,9047) Siteid, ungaged_area, ungaged_slope,
C **     &                 ungaged_shape, Cf2, Cf25, Cf100
        write(17,9047) Siteid, ungaged_area, ungaged_slope,
     &                 ungaged_shape
       endif
     
C-- Warning flag on highly leveraged ungaged sites
 9050 FORMAT (/,2x,'*** WARNING ***********************',/,
     & '  Estimation is OUTSIDE range of observed data.',/,
     & '  Basin characteristics of ungaged site are substanitally ',
     & 'different from ',/,
     & '  the basin characteristics of the selected stations.',/,/,
     & '  ** Specifically, the sampling error variance of estimation ',
     & 'is larger than ',/,
     & '  the maximum sampling error variance of the selected ',
     & 'stations.',/,
     & '  Therefore, the GLS regression model and its estimates ',
     & 'might not be',/,
     & '  appropriate for peak-streamflow frequency estimation for ',
     & 'the ungaged site.',/,/
     & '  ** The user assumes complete responsibility for the use and ',
     & 'interpretation ',/,
     & '     of the presented values.',/,/
     & '  ** However, choosing different station selection criteria ',
     & 'might produce',/,
     & '     a different set of stations, which in turn ',
     & 'might alter',/,
     & '     the regression so that the ungaged site lies WITHIN ',
     & 'range of',/,
     & '     the observed data.  This is a situation where it might be',
     & ' appropriate',/,
     & '      to reconsider the',a5,'year peak streamflow in favor of',
     & ' an',/,
     & "      alternative value from another 'run' of the program.")
        if ( sepu.gt.hmax ) then 
          write(17,9050) Pklabel(IPK)
          write(*,9050)  Pklabel(IPK)
        endif
      write(*,'(1x,70a)')  ('-',i=1,70)
      write(14,'(1x,70a)')  ('-',i=1,70)
      write(17,'(1x,70a)') ('-',i=1,70)
      write(*,*)  ' '
      write(14,*)  ' '
      write(17,*) ' '

      return
      end


      SUBROUTINE SECANT(FX)
      implicit none
      integer MaxSites, MaxInd, MaxDatBase
      parameter (MaxSites = 50, MaxInd = 10, MaxDatBase = 700)
      REAL f1, f2, f3, fnew, x1, x2, x3, xnew
      INTEGER i, j
      REAL Z, Sta, Uv, rhoc, lat, long   
      COMMON /ZZ/ Z(MaxDatBase,18),Sta(MaxSites),Uv(MaxInd),
     +rhoc(MaxSites,MaxSites),lat(MaxSites),long(MaxSites)
C
      REAL X, Xt, Xtxinv, cov
      COMMON /XX1/ X(MaxSites,MaxInd), Xt(MaxInd,MaxSites),
     +XtXinv(MaxInd,MaxInd),
     +cov(MaxSites,MaxSites)
C
      REAL Stano, Y, Wt, E, ET, C1
      COMMON /XY1/ Stano(MaxSites), Y(MaxSites,1),
     +Wt(MaxSites,MaxSites),E(MaxSites,1),ET(1,MaxSites),C1(1,1)
C
      CHARACTER*8 Xlabel, Ylabel, Pklabel
      COMMON /YY1/ Xlabel(8), Ylabel(8), Pklabel(6)
C
      REAL Bhat, Ak, ungaged_area, ungaged_slope, Cf2, Cf25, Cf100
      COMMON /XX2/ Bhat(MaxInd,1), Ak(6), ungaged_area, ungaged_slope,
     +Cf2, Cf25, Cf100
C
      REAL Gamasq, reg_var, Atse, Atscov
      INTEGER Ne, Nsites
      COMMON /YY2/ Gamasq, reg_var, Ne, Nsites, Atse, Atscov
C
      CHARACTER*30 Siteid
      COMMON /YY3/ SITEID
      real FX
      EXTERNAL FX
 
      x1 = 0.0
      x3 = 0.0
      x2 = reg_var*2.
      f2 = FX(x2)
      f1 = FX(x1)
      if ( f1.LT.0. ) then
c
c midpoint search for good starting point
c
        DO 50 j = 1, 3
          xnew = (x1+x2)/2.
          fnew = FX(xnew)
          if ( fnew.LT.0. ) then
            x1 = xnew
            fnew = fnew
          else
            x2 = xnew
            f2 = fnew
          endif
 50     CONTINUE

c
c search for gama sq using secant serch
c
        DO 100 i = 1, 30
          x3 = x1 - f1*(x2-x1)/(f2-f1)
          if ( x3.LT.0. ) then
            x3 = AMIN1(x2,x1)/2.
            f3 = FX(x3)
          else
            f3 = FX(x3)
          endif
          if ( ABS(f3).LT..0001 ) GOTO 200
          if ( ABS(f1).LT.ABS(f2) ) then
            x2 = x3
            f2 = f3
          else
            x1 = x3
            f1 = f3
          endif
          if((f2-f1).eq.0.0) then
            write(*,*) 'STATUS--F2-F1 = 0 about to divide by'
            stop
          endif
 100    CONTINUE
      endif
 200  CONTINUE
      Gamasq = x3
      return
      END




C==================================================  STUTP       =======
      FUNCTION STUTP(X,N)
      REAL a, b, GAUSCF, rhpi, STUTP, t, X, y, z
      INTEGER j, N, nn
C  STUDENT T PROBABILITY
C  STUTP = PROB( STUDENT T WITH N DEG FR  .LT.  X )
C  NOTE  -  PROB(ABS(T).GT.X) = 2.*STUTP(-X,N) (FOR X .GT. 0.)
C  SUBPGM USED - GAUSCF
C  REF - G.W. HILL, ACM ALGOR 395, OCTOBER 1970.
C    USGS - WK 12/79.
      DATA rhpi/ 0.63661977 /
      STUTP = .5
      if ( N.LT.1 ) return
      nn = N
      z = 1.
      t = X**2
      y = t/nn
      b = 1.0 + y
      if ( .NOT.(nn.GE.20.AND.t.LT.nn.OR.nn.GT.200) ) then
C              ( OR if NN NON-INTEGER)
        if ( nn.LT.20 .AND. t.LT.4. ) then
C  --  NESTED SUMMATION OF COSINE SERIES
          y = SQRT(y)
          a = y
          if ( nn.EQ.1 ) a = 0.
        else
C  --  TAIL SERIES FOR LARGE T
          a = SQRT(b)
          y = a*nn
          j = 0
 20       CONTINUE
          j = j + 2
          if ( a.EQ.z ) then
            nn = nn + 2
            z = 0.
            y = 0.
            a = -a
          else
            z = a
            y = y*(j-1)/(b*j)
            a = a + y/(nn+j)
            GOTO 20
          endif
        endif
 50     CONTINUE
        nn = nn - 2
        if ( nn.LE.1 ) then
          if ( nn.EQ.0 ) a = a/SQRT(b)
          if ( nn.NE.0 ) a = (ATAN(y)+a/b)*rhpi
          STUTP = 0.5*(z-a)
          if ( X.GT.0. ) STUTP = 1. - STUTP
          return
        else
          a = (nn-1)/(b*nn)*a + y
          GOTO 50
        endif
      endif
C
C  --  ASYMPTOTIC SERIES FOR LARGE OR NONINTEGER N
      if ( y.GT.1E-6 ) y = ALOG(b)
      a = nn - 0.5
      b = 48.*a**2
      y = a*y
      y = (((((-0.4*y-3.3)*y-24.)*y-85.5)/(0.8*y**2+100.+b)+y+3.)/b+1.)
     &    *SQRT(y)
      STUTP = GAUSCF(-y)
      if ( X.GT.0. ) STUTP = 1. - STUTP
      return
      end


      SUBROUTINE INVERT(N,NDIM,DET,COVINV,COV)
      implicit none
      integer MaxSites
      parameter (MaxSites = 50)
      INTEGER i, im, j, k, N, NDIM
      REAL detl
      REAL DET, sum, temp, COVINV(NDIM,NDIM), COV(NDIM,NDIM), 
     &    B(MaxSites,MaxSites), A(MaxSites,MaxSites)
C --------------------------------------------------------------
C   COV IS AN N*N MATRIX
C   SUBROUTINE COMPUTES DETERMINANT OF COV AS COVINV
C   B IS THE LOWER TRIANGULAR DECOMPOSITION OF COV
C --------------------------------------------------------------
      if ( N.EQ.2 ) then
        DET = COV(1,1)*COV(2,2) - COV(1,2)**2
        temp = COV(1,1)/DET
        COVINV(1,1) = COV(2,2)/DET
        COVINV(2,2) = temp
        COVINV(1,2) = -COV(1,2)/DET
        COVINV(2,1) = COVINV(1,2)
      else
        CALL DECOMP(N,NDIM,COV,B)
        detl = B(1,1)
        DO 50 i = 2, N
          if(detl.gt.5e+19) then
            write(*,*) 'ERROR--Numerical overflow on B(i,i)'
            write(*,*) '       product expansion series.'
            stop
          endif
          detl = detl*B(i,i)
 50     CONTINUE
C-- Following if statement is a questionable fix
        if(detl.gt.5e+19) then
          write(*,*) 'ERR0R--Determinant is too large.'
          write(*,*) '       Numerical overflow.'
          write(*,*) '       Bad fix, but try fewer stations.'
          stop
        else
          DET = detl**2
        endif
C        if ( DET.EQ.0.0 ) then
C          WRITE (6,9010)
C 9010 FORMAT (1X,' PROCESSING STOPPED -- SINGULAR MATRIX')
C          STOP
C        endif
        A(1,1) = 1./B(1,1)
        A(2,2) = 1./b(2,2)
        A(2,1) = -B(2,1)*A(1,1)*A(2,2)
C
        DO 100 i = 3, N
          A(i,i) = 1./B(i,i)
          im = i - 1
          DO 80 k = 1, im
            sum = 0.
            DO 60 j = k, im
              sum = sum + B(i,j)*A(j,k)
 60         CONTINUE
            A(i,k) = -sum*A(i,i)
 80       CONTINUE
 100    CONTINUE
C
        DO 150 i = 1, N
          DO 120 j = 1, i
            sum = 0.
            DO 110 k = i, N
              sum = sum + A(k,i)*A(k,j)
 110        CONTINUE
            COVINV(i,j) = sum
            COVINV(j,i) = sum
 120      CONTINUE
 150    CONTINUE
      endif
      return
      end


      SUBROUTINE DECOMP(N,NDIM,XLAM,B)
      implicit none
      integer MaxSites
      parameter (MaxSites = 50)
      INTEGER is, ism, js, jsm, ks, N, NDIM
      REAL XLAM(NDIM,NDIM), B(MaxSites,MaxSites), bh, bn
C --------------------------------------------------------------
C  CHOLESKY DECOMPOSITION  BB-TRANSPOSE = XLAM
C --------------------------------------------------------------
      if ( XLAM(1,1).LE.0. .OR. XLAM(2,2).LE.0. ) then
        write(*,9005) NDIM,XLAM(1,1),XLAM(2,1),XLAM(2,2),XLAM(1,2)
      endif
      B(1,1) = SQRT(XLAM(1,1))
      B(1,2) = 0.
      B(2,1) = XLAM(2,1)/B(1,1)
      B(2,2) = SQRT(XLAM(2,2)-B(2,1)**2)
C-- If Matrix is 2x2 or 1x1 exit early
      if ( N.LE.2 ) then
        return
      endif
C-- Main decomposition algorithm
      DO 100 is = 3, N
        B(is,1) = XLAM(is,1)/B(1,1)
        bn = XLAM(is,is) - B(is,1)**2
        ism = is - 1
        DO 50 js = 2, ism
          jsm = js - 1
          bh = XLAM(is,js)
          DO 20 ks = 1, jsm
            bh = bh - B(is,ks)*B(js,ks)
 20       CONTINUE
          B(is,js) = bh/B(js,js)
          bn = bn - B(is,js)**2
 50     CONTINUE
        if(bn.le.0.0) write(*,9020) bn
        B(is,is) = SQRT(AMAX1(bn,0.00))
 100  CONTINUE
      return

 9005 FORMAT (' IN DECOMP/ NDIM,XLAM 1-1,2-1,2-2,1-2 = ',I5,4F10.4,/,
     &        ' COVARIANCE MATRIX NOT POSITIVE DEFINITE')
 9020 FORMAT (1X,' COVARIANCE MATRIX NOT POSITIVE DEFINITE BN=',F10.3)
      end
C==================================================




C==================================================  GAUSEX       ======
      FUNCTION GAUSEX(EXPROB)
      REAL ax, c0, c1, c2, CUMPRB, d, d1, d2, d3, EXPROB, GAUSEX, p, pr, 
     &     t, xlim, XX
C
C  GAUSSIAN PROBABILITY FUNCTIONS   W.KIRBY  JUNE 71
C     GAUSEX=VALUE EXCEEDED WITH PROB EXPROB
C     GAUSAB=VALUE (NOT EXCEEDED) WITH PROBCUMPROB
C     GAUSCF=CUMULATIVE PROBABILITY FUNCTION
C     GAUSDY=DENSITY FUNCTION
C  SUBPGMS USED -- NONE
C
C  GAUSCF MODIFIED 740906 WK -- REPLACED ERF FCN REF BY RATIONAL APPRX N
C    ALSO REMOVED DOUBLE PRECISION FROM GAUSEX AND GAUSAB.
C  76-05-04 WK -- TRAP UNDERFLOWS IN EXP IN GUASCF AND DY.
C
C
      DATA xlim/18.3/
      DATA c0, c1, c2/2.51551700, .8028530000, .0103280000/
      DATA d1, d2, d3/1.432788000, .1892690000, .0013080000/
C
      p = EXPROB
 100  CONTINUE
      if ( p.GE.1.0 ) then
        GAUSEX = -10.
      elseif ( p.GT.0. ) then
        pr = p
        if ( p.GT..5 ) pr = 1.00 - pr
        t = SQRT(-2.00*ALOG(pr))
        GAUSEX = t - (c0+t*(c1+t*c2))/(1.0+t*(d1+t*(d2+t*d3)))
        if ( p.GT..5 ) GAUSEX = -GAUSEX
      else
        GAUSEX = +10.
      endif
      return
C
      ENTRY GAUSAB(CUMPRB)
      GAUSAB = 0.
      p = 1. - CUMPRB
      GOTO 100
C
      ENTRY GAUSCF(XX)
      ax = ABS(XX)
      GAUSCF = 1.
      if ( ax.LE.xlim ) then
        t = 1.0/(1.0+.2316419*ax)
        d = 0.3989423*EXP(-XX*XX*.5)
        GAUSCF = 1. - 
     &           d*t*((((1.330274*t-1.821256)*t+1.781478)*t-0.3565638)
     &           *t+0.3193815)
      endif
      if ( XX.LT.0 ) GAUSCF = 1. - GAUSCF
      return
C
      ENTRY GAUSDY(XX)
      GAUSDY = 0.
      if ( ABS(XX).GT.xlim ) return
      GAUSDY = .3989423*EXP(-.500*XX*XX)
      return
      END
c



      SUBROUTINE CFX(ALTD,ALTM,ALTS,ALND,ALNM,ALNS,C2,C25,C100)
      implicit none
      REAL ALND, ALNM, ALNS, ALTD, ALTM, ALTS, C100, C25, C2, sx, sy
      INTEGER i, ix, iy, j, npts
c subroutine computes climate factors from lat and long
c
C     COMPUTES X,Y COORDINATES(KILOMETERS) BASED ON LAT AND LONG,
C     USING ALBERS EQUAL-AREA CONIC PROJECTION--WITH EARTH AS ELLIPSOID.
C     EQUATIONS FROM BULLETIN 1532, PAGES 96-99.
C     A = a, equatorial radius of ellipsoid
C     E2 = e*2, square of eccentricity of ellipsoid (0.006768658).
C     PHI1,PHI2 = standard parallels of latitude (29.5,45.5).
C     PHI0 = middle latitude, or latitude chosen as the origin of y-coordinate (38.0).
C     LAM0 = central meridian of the map, or longitude chosen as the origin
C            of x-coordinate (96.0).
C     PHI = latitude of desired y-coordinate.
C     LAM = longitude of desired x-coordinate.
C     XOFF = offset to x-coordinate to make all positive for KRIGING.
C     YOFF = offset to y-coordinate to make all positive for KRIGING.
      REAL C2a(30,26), C25a(30,26), C100a(30,26)
      DOUBLE PRECISION q, q0, q1, q2, phi, phi0, phi1, phi2, e, e2, m1, 
     &                 m2, n, rho, rho0, a, c, theta, lam, lam0, x, y, 
     &                 xoff, yoff, dp, mp, sp, dm, mm, sm, one, two, tpi
      DATA a/6378206.4/, e2/0.006768658/, tpi/6.2831853/
      DATA phi1/29.5/, phi2/45.5/, phi0/38.0/, lam0/96.0/

C   ZERO ARRAYS THEN READ IN AVAILABLE KRIGED CLIMATE FACTORS
      DO 100 i = 1, 30
        DO 50 j = 1, 26
          C2a(i,j) = 0.0
          C25a(i,j) = 0.0
          C100a(i,j) = 0.0
 50     CONTINUE
 100  CONTINUE
      OPEN(8,file='cgrid.krg',status='OLD',err=998)
      READ(8,9005) npts
      DO 200 i = 1, npts
        READ (8,9010) ix, iy, C2, C25, C100
        C2a(ix,iy) = C2
        C25a(ix,iy) = C25
        C100a(ix,iy) = C100
 200  CONTINUE
      close(8)

C-- COMPUTE CLIMATIC FACTORS FOR THE UNGAGED SITE
      xoff = 1000.0D0
      yoff = 1500.0D0
      one = 1.0D0
      two = 2.0D0
      e = DSQRT(e2)
      phi0 = phi0*tpi/360.D0
      phi1 = phi1*tpi/360.D0
      phi2 = phi2*tpi/360.D0
      q0 = (one-e2)*(DSIN(phi0)/(one-e2*DSIN(phi0)*DSIN(phi0))
     &     -(one/(two*e))*DLOG((one-e*DSIN(phi0))/(one+e*DSIN(phi0))))
      q1 = (one-e2)*(DSIN(phi1)/(one-e2*DSIN(phi1)*DSIN(phi1))
     &     -(one/(two*e))*DLOG((one-e*DSIN(phi1))/(one+e*DSIN(phi1))))
      q2 = (one-e2)*(DSIN(phi2)/(one-e2*DSIN(phi2)*DSIN(phi2))
     &     -(one/(two*e))*DLOG((one-e*DSIN(phi2))/(one+e*DSIN(phi2))))
      m1 = DCOS(phi1)/DSQRT(one-e2*DSIN(phi1)*DSIN(phi1))
      m2 = DCOS(phi2)/DSQRT(one-e2*DSIN(phi2)*DSIN(phi2))
      n = (m1*m1-m2*m2)/(q2-q1)
      c = m1*m1 + n*q1
      rho0 = a*DSQRT(c-n*q0)/n
      dp = DBLE(ALTD)
      mp = DBLE(ALTM)
      sp = DBLE(ALTS)
      dm = DBLE(ALND)
      mm = DBLE(ALNM)
      sm = DBLE(ALNS)
      mp = mp + sp/60.D0
      phi = (dp+mp/60.D0)*tpi/360.D0
      mm = mm + sm/60.D0
      lam = (dm+mm/60.D0)
      q = (one-e2)*(DSIN(phi)/(one-e2*DSIN(phi)*DSIN(phi))-(one/(two*e))
     &    *DLOG((one-e*DSIN(phi))/(one+e*DSIN(phi))))
      theta = n*(lam0-lam)*tpi/360.D0
      rho = a*DSQRT(c-n*q)/n
      x = rho*DSIN(theta)/1000.D0 + xoff
      y = (rho0-rho*DCOS(theta))/1000.D0 + yoff
      sx = SNGL(x)
      sy = SNGL(y)
      CALL WGT(sx,sy,C2,C25,C100,C2a,C25a,C100a)
      return
 9005 FORMAT (I4)
 9010 FORMAT (I8,I9,3F15.0)
 998  write(*,*) "ERROR--Could not open 'cgrid.krg'"
      stop
      END




      SUBROUTINE WGT(X,Y,C2,C25,C100,C2a,C25a,C100a)
      implicit none
      REAL C100, C25, C2, cp, r1, r2, r3, r4, rx, ry, 
     &     wr1, wr2, wr3, wr4, X, xr, Y, yr
      INTEGER ix, ixp, iy, iyp
      REAL C2a(30,26), C25a(30,26), C100a(30,26)
      wr1 = 0.0
      wr2 = 0.0
      wr3 = 0.0
      wr4 = 0.0
C  ESTIMATE CLIMATE FACTOR FROM INTERPOLATION OF KRIGED VALUES AT 4 POINTS
C  SURROUNDING X,Y LOCATION OF INTEREST.  HOWEVER, USE KRIGED VALUE IF
C  DISTANCE TO NEAREST POINT IS 10KM OR LESS.
      ix = X/100.0
      rx = ix
      xr = X - 100.0*rx
      iy = Y/100.0
      ry = iy
      yr = Y - 100.0*ry
      ix = ix + 1
      iy = iy + 1
      r1 = SQRT(xr**2+yr**2)
      if ( r1.LE.10. ) then
        wr1 = 1.0
        GOTO 100
      endif
      r2 = SQRT(xr**2+(100.0-yr)**2)
      if ( r2.LE.10. ) then
        wr2 = 1.0
        GOTO 100
      endif
      r3 = SQRT((100.0-xr)**2+(100.0-yr)**2)
      if ( r3.LE.10. ) then
        wr3 = 1.0
        GOTO 100
      endif
      r4 = SQRT((100.0-xr)**2+yr**2)
      if ( r4.LE.10. ) then
        wr4 = 1.0
        GOTO 100
      endif
      cp = 1.0/(1.0/r1+1.0/r2+1.0/r3+1.0/r4)
      wr1 = cp/r1
      wr2 = cp/r2
      wr3 = cp/r3
      wr4 = cp/r4
 100  CONTINUE
      ixp = ix + 1
      iyp = iy + 1
      C2   = wr1*C2a(ix,iy) + wr2*C2a(ix,iyp) + wr3*C2a(ixp,iyp)
     &       + wr4*C2a(ixp,iy)
      C25  = wr1*C25a(ix,iy) + wr2*C25a(ix,iyp) + wr3*C25a(ixp,iyp)
     &       + wr4*C25a(ixp,iy)
      C100 = wr1*C100a(ix,iy) + wr2*C100a(ix,iyp) + wr3*C100a(ixp,iyp)
     &       + wr4*C100a(ixp,iy)
      return
      END





      SUBROUTINE MLTPLY(PROD,X,Y,K1,K2,K3,N1,N2,N3)
      implicit none
      INTEGER i, j, k, K1, K2, K3, N1, N2, N3
      REAL PROD(N1,K3), X(N2,K2), Y(N3,K3), sum
C --------------------------------------------------------------
C  X IS K1*K2 MATRIX
C  Y IS K2*K3 MATRIX
C  PROD = X*Y IS A K1*K3 MATRIX
C --------------------------------------------------------------
      DO 100 i = 1, K1
        DO 50 k = 1, K3
          sum = 0.
          DO 20 j = 1, K2
            sum = sum + X(i,j)*Y(j,k)
 20       CONTINUE
          PROD(i,k) = sum
 50     CONTINUE
 100  CONTINUE
      return
      END
c


      subroutine screen(isc)    
      character*4 word, f
      dimension word(20)
      if(isc.eq.1) f='sc.1'
      if(isc.eq.2) f='sc.2'
      if(isc.eq.3) f='sc.3'
      open(18,file=f)
      do 1, i=1,23
        read(17,2)word
        write(*,2)word
    1 continue
      close(18)
    2 format(20a4)
      return
      end



      subroutine dist_correlation(num,theta,alpha,lat,long,rhoc)
      implicit none
      integer MaxSites,num
      parameter (MaxSites = 50)
      real theta,alpha,dist
      real lat(1:MaxSites),long(1:MaxSites)
      real latx,laty,longx,longy
      real rhoc(1:MaxSites,1:MaxSites)
      integer i,j
c
      if (theta.lt.0.or.theta.ge.1.) then
         print *,'ERROR-- Theta must be between 0 and 1'
         stop
      endif
c
      do 20 i=1,num
        do 30 j=1,num
          latx  = lat(i)
          laty  = lat(j)
          longx = long(i)
          longy = long(j)
          call TASKER_DISTANCE(latx,laty,longx,longy,dist)
          rhoc(i,j) = dist
   30   continue
   20 continue
c
      do 61 i=1,num
        do 60 j=1,i
          if (i.eq.j)then
             rhoc(i,j)=1.0
          else
             rhoc(i,j)=theta**(rhoc(i,j)/(alpha*rhoc(i,j)+1.))
             if(rhoc(i,j).lt.0.0) then
               write(*,*) 'rhoc(i,j) = ',i,j,rhoc(i,j)
               write(*,*) 'ERROR--Cholesky Decomp requires >= 0'
               stop
             endif
          endif
  60    continue
  61  continue
      return
      end

      subroutine TASKER_DISTANCE(latxin,latyin,longxin,longyin,dist)
      implicit none
C-- The latitudes and longitudes are inputted in decimal degrees
C-- Distance, dist, is returned in miles
C Date: Thu, 03 Sep 1998 12:48:20 -0400
C From: Gary D Tasker <gdtasker@usgs.gov>
C To: William Asquith <wasquith@nwistx.cr.usgs.gov>
C Subject: Re: GLS
C Will,
C The projection is Albers Equal-Area Conic Projection (with Earth as an
C elipsoid) Ref. Snyder, J.P., 1982, "Map Projections Used by The U.S.
C Geological Survey", U. S. Geological Survey Bulletin 1532, p. 96-99.
C Gary
      real latx,laty,longx,longy,dist,latxin,latyin,longxin,longyin
      real pi,radian,a,e2,latdiff,longdiff,latave,op1,op2,op3 
      parameter (pi = 3.14159265, a = 6378206, e2 = 0.00676866) 
      radian = pi/180
      latx = latxin * radian
      laty = latyin * radian
      longx = longxin * radian
      longy = longyin * radian
    
      latdiff = abs(latx-laty)
      longdiff = abs(longx-longy)
      latave = (latx + laty)/2.
      op1 = a/sqrt(1.-e2*sin(latave)**2)
      op2 = ((1.-e2)**2 * (latdiff)**2)
      op2 = op2 / ((1.-e2 * sin(latave)**2)**2)
      op3 = cos(latave)**2 * longdiff**2
c-- Convert distance to miles by dividing the meters by 1000 to get
c    kilometeres and then convert to miles by multiplying by 0.621
      dist = .62137119*(op1 * sqrt(op2+op3))/1000.  
      return
      end

      subroutine STARTUP(Siteid,altd,altm,alts,alnd,alnm,alns,
     &                   ungaged_area,ungaged_shape,ungaged_slope,
     &                   ungaged_map,ungaged_P224,Nsites,alpha,
     &                   theta,Cf2,Cf25,Cf100,similar_response,
     &                   Ne_response,alpha_area,metric)
      implicit none
      character Siteid*30,response*1,metric*1
      character fulloutput*30,summaryoutput*30,selectionfile*30
      real altd,altm,alts,alnd,alnm,alns
      real ungaged_area,ungaged_slope,ungaged_shape
      real ungaged_map,ungaged_P224,alpha,theta     
      real Cf2,Cf25,Cf100,alpha_area
      integer Nsites,similar_response,Ne_response,warning_response

      print *,'   '
      print *,'   '
      print *,'   '
      print *,' -------------------------------------------------------'
      print *,'     TEXAS SITE-SPECIFIC FLOOD FREQUENCY PROGRAM'
      print *,'                   March 2003'
      print *,'        U.S. Geological Survey, Austin, Texas'
      print *,'          developed in cooperation with the'
      print *,'         Texas Department of Transportation'
      print *,'               '
      print *,' This program computes estimates of T-year floods'
      print *,' for ungaged stream sites within natural basins in'
      print *,' Texas based on a non-fixed boundary generalized least-'
      print *,' squares multivariate regression.'
      print *,' The user enters a site identifier, latitude and longi-'
      print *,' tude, drainage area and other basin characteristics'
      print *,' for the ungaged site.  The program selects gaged sites'
      print *,' with similar basin characteristics and performs a'
      print *,' multivariate generalized least-squares regression'
      print *,' analysis.  The program has been intentially setup to '
      print *,' require both basin shape factor AND stream slope if '
      print *,' either of these values is available to the user.'
      print *,' '
      print *,' BUG REPORTS: wasquith@usgs.gov'
      print *,' -------------------------------------------------------'
      print *,'                                                 ' 
 5    write(*,*) 'Selection Option: '
      write(*,*) '   Warning No. 1 --> (1)  Different T-year Discharges'
      write(*,*) '   Warning No. 2 --> (2)  Use of Resulting Equation'
      write(*,*) 'Continue Program --> (3)          ---'
      read(*,*) warning_response
      if(warning_response.eq.1) then
C-- WARNING FLAG ON DISCHARGE CALCULATION
        print *,' '
        print *,' -----------------------------------------------------'
        print *,' WARNING:  The user is given several choices on'
        print *,"  how the most 'similar' stations are selected.  The" 
        print *,'  various selection criteria will often produce'
        print *,'  DIFFERENT, but all equally valid peak-streamflow'
        print *,'  estimates.  The user must use engineering and'
        print *,'  hydrologic judgement on how to choose the most'
        print *,"  'similar' stations.  It is very important that "
        print *,'  after a selection choice is made, the subsequent' 
        print *,'  regression and peak-streamflow frequency estimate'
        print *,'  should be considered the ONLY valid one.  It is'
        print *,'  inappropriate to pick and choose the number of'   
        print *,'  stations and (or) the selection criteria in order'
        print *,"  to 'get' the discharge that the user desires."
        print *,' -----------------------------------------------------'
        print *,' '
        goto 5
      elseif(warning_response.eq.2) then
        print *,' '
        print *,' -----------------------------------------------------'
        print *,' WARNING:  Because of the nature by which stations'
        print *,'  are selected, the equation is considered ONLY valid' 
        print *,'  for the particular ungaged_site.  Transferring and'
        print *,'  using the equation for ungaged sites having even'
        print *,'  slightly different basin characteristics could'
        print *,'  potentially produce erroneous peak-streamflow'
        print *,'  frequency values.  This fact differs from the more'
        print *,'  traditional regional regression equation approach,'
        print *,'  where an equation is developed such that it is'
        print *,'  transferable to a broad range of stream sites'
        print *,'  within the region.'
        print *,' '
        print *,'  The authors do not see an simple means by which to'
        print *,'  identify which other ungaged sites that a previously' 
        print *,'  generated equation can be safely used on.  Thus,'   
        print *,'  the authors suggest that, unless ungaged sites are'
        print *,'  along the same stream and within reasonable stream'
        print *,'  distances from one another, a separate site-specific'
        print *,'  peak-streamflow frequency analysis using the program'
        print *,"  be performed for the 'new' ungaged site."
        print *,' -----------------------------------------------------'
        print *,' '
        goto 5     
      endif


      print *,' Enter file name for FULL output: '
      read(*,'(a)') fulloutput
c      fulloutput = 'glsregress.out'
      print *,' Enter file name for SUMMARY output: '
      read(*,'(a)') summaryoutput
c      summaryoutput = 'gls_sum.out'
      print *,' Enter file name for STATION DATA output: '
      read(*,'(a)')  selectionfile
c      selectionfile = 'gls_stat.out'
      open(11,file=selectionfile,err=989)
      open(14,file=summaryoutput,err=990)
      open(17,file=fulloutput,err=991)
      print *,' English or Metric units (E/M)?'
      read(*,'(a1)') metric
      if(metric.eq.'M'.or.metric.eq.'m') then
        metric = 'M'
        print *,' Enter site id (up to 30 characters)'
        READ (*,'(A30)') Siteid
        print *,' Enter site latitude (dd mm ss)'
        READ (*,*) altd, altm, alts
        print *,' Enter site longitude (dd mm ss)'
        READ (*,*) alnd, alnm, alns
        print *,' Enter drainage area, shape, slope (km^2, --, m/km)'
        print *,'  (enter a value less than or equal to zero if',
     &          ' unknown)'
        READ (*,*) ungaged_area,ungaged_shape,ungaged_slope 
c        print *,' Enter mean annual and 2-year 24-hr precipitation',
c     &   ' (mm/yr; mm/day)'
c        READ (*,*) ungaged_map,ungaged_P224
        ungaged_map  =  -9999
        ungaged_P224 = -9999
        if(ungaged_area.le.0) then
          write(*,*) 'ERROR--Must have >= 0 drainage area'
          stop
        endif
      else 
        metric = 'E'
        print *,' Enter site id (up to 30 characters)'
        READ (*,'(A30)') Siteid
c        Siteid = 'Guadalupe Comfort'
        print *,' Enter site latitude (dd mm ss)'
        READ (*,*) altd, altm, alts
c       altd = 29
c       altm = 58
c       alts = 10
       print *,' Enter site longitude (dd mm ss)'
       READ (*,*) alnd, alnm, alns
c       alnd = 98
c       alnm = 53
c       alns = 33
       print *,' Enter drainage area, shape, slope (mi^2, --, ft/mi)'
       print *,'  (enter a value less than or equal to zero if',
     &         ' unknown)'
       READ (*,*) ungaged_area,ungaged_shape,ungaged_slope 
c       ungaged_area = 839
c       ungaged_shape = 5.65
c       ungaged_slope = 15.01
        if(ungaged_area.le.0) then
          write(*,*) 'ERROR--Must have >= 0 drainage area'
          stop
        endif 


c       print *,' Enter mean annual and 2-year 24-hr precipitation',
c     &' (in/yr; in/day)'
c       READ (*,*) ungaged_map,ungaged_P224
       ungaged_map  =  -9999
       ungaged_P224 = -9999
      endif

C SUMMARIZE THE SITES CHARACTERISTICS
 9010 format(a,f4.0,'d ',f3.0,'m ',f3.0,'s ')
      write(*,*) ' '
      write(*,*) ' Summary of the ungaged site: ',Siteid
      write(*,9010) '    Latitude: ',altd,altm,alts
      write(*,9010) '   Longitude: ',alnd,alnm,alns
      write(*,*) '  Drainage Area        = ',ungaged_area
      write(*,*) '  Basin-Shape Factor   = ',ungaged_shape
      write(*,*) '  Stream Slope         = ',ungaged_slope
      write(*,*) ' '
      write(14,*) ' SUMMARY OF SITE-SPECIFIC FLOOD FREQUENCY'
      write(14,*) '  Summary of the ungaged site: ',Siteid
      write(14,9010) '     Latitude: ',altd,altm,alts
      write(14,9010) '    Longitude: ',alnd,alnm,alns
      write(14,*) '   Drainage Area        = ',ungaged_area
      write(14,*) '   Basin-Shape Factor   = ',ungaged_shape
      write(14,*) '   Stream Slope         = ',ungaged_slope
      write(14,*) ' '


 31   print *," Enter no. of stations to 'pool' or regionalize together"
      read (*,*,err=31) Nsites
      if(Nsites.lt.20) then
        write(*,*) '  Nsites too small, resetting to Nsites = 20' 
        Nsites = 20
      endif
      if(Nsites.gt.50) then
        write(*,*) '  Nsites too large, resetting to Nsites = 50'
        Nsites = 50
      endif
      write(14,*) ' ANALYSIS CRITERIA'
      write(14,*) '  Number of stations = ',Nsites
      write(14,*) ' '


      if(ungaged_slope.le.0.or.ungaged_shape.le.0) then
        write(*,*) ' NOTICE'
        write(*,*) ' Basin-shape factor and stream slope are unknown.'
        write(*,*) ' Regression will only include DRAINAGE AREA'
        Ne_response = 2
      else
        write(*,*) ' '
        print *,' Use only DRAINAGE AREA otherwise SLOPE and SHAPE'
        print *,'  will also be considered (Y for AREA/ N for ALL)'
        read(*,'(a1)') response
        if(response.eq.'y'.or.response.eq.'Y') then
          print *,' Program considering only drainage area'
          write(14,*) '     Program considering only drainage area'
          Ne_response = 2 
        else
          print *, ' Program considering AREA, SHAPE, and SLOPE'
          write(14,*) '  Program considering AREA, SHAPE, and SLOPE'
          Ne_response = 4
        endif
      endif
      write(14,*) ' '
      print *,' '
      print *,' Station Selection Criteria (see text for more details):'
      print *,'  (1)  PROXIMITY: Drainage Area bounded by 1.5 ',
     &        'log10 cycles'
      print *,'  (2) SIMILARITY: Drainage Area, Basin Shape,',
     &        ' Stream Slope, and Distance'
      print *,'  (3) SIMILARITY: Drainage Area and Distance'
 44   print *,' Enter (1,2,3) or (1,3 only if no shape or slope): '
      read(*,*) similar_response
      if(similar_response.ge.1.and.similar_response.le.3) then
        continue
      else
        write(*,*) 'ERROR--Invalid similarity.  Please reselect.'
        goto 44
      endif
      if(ungaged_slope.le.0.or.ungaged_shape.le.0) then
        if(similar_response.eq.2) then
          write(*,*) 'ERROR--Invalid similarity choice when either',
     &               ' shape and slope are unknown.'
          write(*,*) '    Select options (1 or 3)'
          write(*,*) ' '
          goto 44
        endif
      endif
      if(similar_response.eq.1) then
        write(14,*) '  Selection based on (1) PROXIMITY'
      elseif(similar_response.eq.2) then
        write(14,*) '  Selection based on (2) SIMILARITY using'
        write(14,*) '                            Drainage Area'
        write(14,*) '                            Basin-Shape Factor'
        write(14,*) '                            Stream Slope'
        write(14,*) '                            Distance'
      else
        write(14,*) '  Selection based on (3) SIMILARITY using '
        write(14,*) '                            Drainage Area'
        write(14,*) '                            Distance'
      endif
      write(14,*) ' '
      alpha_area = 1.5


C-- COMPUTE THE CLIMATE FACTORS FOR THE UNGAGED SITE
      CALL CFX(altd,altm,alts,alnd,alnm,alns,Cf2,Cf25,Cf100)
C ** DISABLED FOR CLIMATE FACTOR REMOVAL
c      write(*,*) '  Mean annual precip.  = ',ungaged_map
c      write(*,*) '  2-yr 24-hour precip. = ',ungaged_P224
c      write(*,*) ' Climate Factors'
c      write(*,'(2x,3(a,f6.3,3x))') ' Cf2 = ',Cf2,
c     &                             ' Cf25 = ',Cf25,
c     &                             ' Cf100 = ',Cf100

C-- Parameters of Tasker's cross-covariance model determined for Texas
C    by Asquith and Slade in 1996.
      alpha=.0025
      theta=.983

      return
 989  write(*,*) 'ERROR--Could not open selected stations file ',
     &            selectionfile
      stop
 990  write(*,*) 'ERROR--Could not open selected stations file ',
     &            summaryoutput
      stop
 991  write(*,*) 'ERROR--Could not open full output file ',
     &            fulloutput
      stop
      end


      subroutine PRELIMINARIES(total_stations,Z,Mcon,ind_var_sd,
     &                         metric)
      implicit none
      integer MaxDatBase
      parameter (MaxDatBase = 700)
      character HEADER*160,metric*1
      real Mcon(MaxDatBase,MaxDatBase),Z(MaxDatBase,18)
      real ind_var_sd(1:8),s,ss,latd,latm,lats,lond,lonm,lons
      integer i,j,k,total_stations
c Open files and read in data

 9001 format(15x,a)
 9002 format(19x,a)
      write(17,'(1x,70a)') ('-',i=1,70)                         
      write(17,'(14x,a)') 'TEXAS SITE-SPECIFIC FLOOD FREQUENCY PROGRAM'
      write(17,'(28x,a)') 'version March 2003'
      write(17,'(17x,a)') 'U.S. Geological Survey, Austin, Texas'
      write(17,'(19x,a)') 'developed in cooperation with the'
      write(17,'(18x,a)') 'Texas Department of Transportation'
      write(17,*) '   '
      write(17,*) 'Program is part of report:'
      write(17,*) 'Asquith, W.H., and Slade, R.M., 1999, Site-specific ',
     &'estimation of peak-'
      write(17,*) '  streamflow frequency using generalized-least ',
     &'squares regression'
      write(17,*) '  for natural basins in Texas: '
      write(17,*) '  U.S. Geological Survey Water Resources',
     &' Investigations'
      write(17,*) '  Report 99-4172, 19 p.'
      write(17,*) ' '
      write(17,*) 'Data base derived from:'
      write(17,*) 'Asquith, W.H., and Slade, R.M., 1997, Regional ',
     &'equations for'
      write(17,*) '  estimation of peak streamflow frequency for ',
     &'natural basins in'
      write(17,*) '  Texas: USGS Water Resources Investigations ',
     &'Report 96-4307, 68 p.' 
      write(17,*) ' '
      write(17,*) 'Computation algorithms based on those provided by ',
     &' Gary D.'
      write(17,*) 'Tasker, USGS in 1995, 1996 and 1998'
      write(17,'(1x,70a)') ('-',i=1,70)

      write(17,9045)
      write(*,9045)
 9045 format(/,' IMPORTANT NOTE--',/,
     & ' It is important that all the peak-streamflow frequency should',
     & ' be based on the',/,
     & '  same execution of the program using the same assumptions, ',
     & 'in order to',/,
     & '  minimize the possibility of producing discharge values that',
     & ' would not be',/,
     & '  appropriately proportional to their recurrence interval.',/,
     & ' Also, it is inappropriate to select the T-year estimate from',
     & ' one run of the',/,
     & '  program and a different T-year estimate from another run of ',
     & 'the program.',/,
     & ' By using all T-year estimates from one run, the user is ',
     & 'unlikely to',/,
     & '  encounter the situation in which the streamflow value for ',
     & 'one recurrence',/,
     & '  interval is LARGER than the streamflow value for a HIGHER ',
     & 'recurrence',/,
     & '  interval.',/)

      if(metric.eq.'M') then
        write(17,*) '  UNITS IN METRIC (see startup for details)'
      else
        write(17,*) '  UNITS IN ENGLISH (see startup for details)'
      endif
C-- READ IN THE DATA BASE
C-- ORDER OF THE DATA INPUT FILE:
C    Station | Latitude | Longitude | Years | Drainage Area |
C    Mean Annual Precip. | 2-year 24-hour Precip | Stream Slope |
C    Basin Shape | Q2 | Q5 | Q10 | Q25 | Q50 | Q100 | C2 | C25 | C100
      open(10,file='tx664.dat',status='OLD',err=988)

      total_stations = 0
      read(10,'(a160)') HEADER
      do i = 1, MaxDatBase
        read(10,*,end=300) Z(i,1),latd,latm,lats,lond,lonm,lons,
     &    (Z(i,j),j=4,18)
C-- Unit conversions
          Z(i,2) = latd+(latm+(lats/60.))/60.
          Z(i,3) = lond+(lonm+(lons/60.))/60.

C-- Changes units only if metric is desired, input in English
        if(metric.eq.'M') then
C    Convert square miles into square kilometers
          Z(i,5) = log10((10**Z(i,5))*2.58998811)
C    Convert inches into milimeters
          Z(i,6) = log10((10**Z(i,6))*25.4)
          Z(i,7) = log10((10**Z(i,7))*25.4)
C    Convert feet per mile to meters per kilometer
          Z(i,8) = log10((10**Z(i,8))*0.3048/1.609344)
C    Convert discharges in cfs to cms
          do k = 10,15
            Z(i,k) = log10((10**Z(i,k))*0.02831685)
          enddo
        endif
C-- End of unit conversions

        total_stations = total_stations + 1

      enddo
 300  continue 

      write(*,*) total_stations,' stations were read in'
      close(10)
      open(18,file='rec664.txt',status='OLD',err=990)
      do 100 i=1,total_stations
        read(18,9010) (Mcon(i,j),j=1,i)
        do 200 j=1,i     
          Mcon(j,i)=Mcon(i,j)
 200   continue
 100  continue
      close(18)
      write(*,*) 'The concurrent record matrix has been filled'

      do 400 j = 1, 8
        s = 0.0
        ss = 0.0
        do 500 i = 1, total_stations
          if(j.le.5) then
            s = s + Z(i,j+4)
          else
            s = s + Z(i,j+10)
          endif
 500    continue
        ind_var_sd(j) = s / total_stations
        do 600 i = 1, total_stations
          if(j.le.5) then
            ss = ss + (Z(i,j+4)-ind_var_sd(j))**2
          else
            ss = ss + (Z(i,j+10)-ind_var_sd(j))**2
          endif   
 600    continue
        ind_var_sd(j) = (ss / ( total_stations - 1 ))**0.5
 400  continue
      write(*,*) "Std Deviations of independent variables calculated"
       return
 9005 format(3f10.4,f5.0,14f10.4)
 9010 format(20f4.0)

 988  write(*,*) "ERROR--Could not open 'tx664.dat'"
      stop
 990  write(*,*) "ERROR--Could not open 'rec664.txt'"
      stop

      end


C==================================================  STUTX  
      FUNCTION STUTX(P,N) 
C  
C  STUDENT T QUANTILES --             
C   STUTX(P,N) = X SUCH THAT PROB(STUDENT T WITH N D.F. .LE. X) = P.
C  NOTE - ABS(T) HAS PROB Q OF EXCEEDING STUTX( 1.-Q/2., N ). 
C   NOTE -      IER - ERROR FLAG --  1 = F.LT.1.,                      
C                                   2 = P NOT  IN (0,1),  3 = 1+2        
C   SUBPGMS USED -- GAUSAB     (GAUSSIAN ABSCISSA)     
C  REF - G. W. HILL (1970) ACM ALGO 396.  COMM ACM 13(10)619-20.        
C             REV BY WKIRBY 10/76. 2/79.  10/79.                     
C                                                                 
      DATA HPI / 1.5707963268/                                
      SIGN=1.
      IF(P.LT.0.5) SIGN=-1.              
      Q=2.*P                 
      IF(Q.GT.1.) Q=2.*(1.-P)      
      IF(Q.LT.1.) GOTO 1010
      STUTX = 0.
      RETURN    
                                       
1010  CONTINUE                                                
      FN = N    
      IF(N.GE.1 .AND. Q.GT.0. .AND. Q.LT.1.) GOTO 10
      IER = 3
      IF(N.GE.1) IER = 2                                                 
      STUTX = SIGN*1E38                                               
      RETURN                                                      
C                                                              
   10 IF(N.NE.1) GOTO 20                                   
C  -- 1 DEG FR - EXACT          
      STUTX=SIGN/TAN(HPI*Q)
      RETURN
C  
   20 IF(N.NE.2) GOTO 30                                              
C  -- 2 DEG FR - EXACT                                              
      STUTX = SQRT(2.0/(Q*(2.0-Q))-2.0) * SIGN                    
      RETURN                                                   
C                                                           
C  -- EXPANSION FOR N .GT. 2                
   30 A = 1.0/(FN-0.5)                            
      B= 48.0/(A*A)
      C=((20700.*A/B-98.)*A-16.)*A+96.36
      D =  ((94.5/(B+C)-3.)/B+1.)*SQRT(A*HPI)*FN                       
      X = D*Q                                                      
      Y = X**(2.0/FN)                                     
      IF (Y .GT. A+.05) GO TO 40                             
      Y = ((1.0/(((FN+6.)/(FN*Y)-0.089*D-0.822)*(FN+2.)*3.)+ 
     $   0.5/(FN+4.))*Y-1.)*(FN+1.)/(FN+2.)+1.0/Y
      STUTX = SQRT(FN*Y) * SIGN                                         
      RETURN                                                          
C   -- ASYMPTOTIC INVERSE EXPANSION ABOUT NORMAL
   40 X = GAUSAB(0.5*Q)
      Y =  X*X                                                          
      IF (FN .LT. 5.) C = C+0.3*(FN-4.5)*(X+0.6)                   
      C = (((.05*D*X-5.)*X-7.)*X-2.)*X+B+C                      
      Y = (((((0.4*Y+6.3)* Y+36.)*Y+94.5)/C-Y-3.)/B+1.)*X     
      X = A*Y**2
      Y = X + 0.5*X**2
      IF (X .GT. .002) Y = EXP(X) - 1.0                                  
      STUTX = SQRT(FN*Y) * SIGN                                 
      RETURN                                                     
      END
C==================================================                      

